@@ -29,8 +29,7 @@
 #include <string.h>
 #include <math.h>
 
-#include <libart_lgpl/art_misc.h>
-#include <libart_lgpl/art_pixbuf.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include <gtk/gtksignal.h>
 
@@ -152,13 +151,20 @@ gpp_clip (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 	return 1;
 }
 
+static void
+gpp_image_free_pix (guchar *pixels, gpointer data)
+{
+	g_free (pixels);
+
+}
+
 static int
 gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint w, gint h, gint rowstride, gint ch)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomeCanvasGroup * group;
 	GnomeCanvasItem *item;
-	ArtPixBuf *pixbuf;
+	GdkPixbuf *pixbuf;
 	int size, bpp;
 	void *dup;
 	
@@ -173,15 +179,19 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 	}
 	
 	size = w * h * bpp;
-	dup = art_alloc (size);
+	dup = g_malloc (size);
 	if (!dup) return -1;
 
 	if (ch == 3) {
 		memcpy (dup, px, size);
-		pixbuf = art_pixbuf_new_rgb (dup, w, h, rowstride);
+		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
+						   FALSE, 8, w, h, rowstride,
+						   gpp_image_free_pix, NULL);
 	} else if (ch == 4) {
 		memcpy (dup, px, size);
-		pixbuf = art_pixbuf_new_rgba (dup, w, h, rowstride);
+		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
+				                   TRUE, 8, w, h, rowstride,
+						   gpp_image_free_pix, NULL);
 	} else if (ch == 1) {
 		const char *source;
 		char *target;
@@ -198,7 +208,9 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 				source++;
 			}
 		}
-		pixbuf = art_pixbuf_new_rgb (dup, w, h, rowstride * 3);
+		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
+						   FALSE, 8, w, h, rowstride * 3,
+						   gpp_image_free_pix, NULL);
 	} else
 		return -1;
 
