@@ -38,10 +38,6 @@ static void gpa_settings_selector_destroy (GtkObject *object);
 
 static gint gpa_settings_selector_construct (GPAWidget *widget);
 
-static void gpa_ss_root_modified (GPANode *config, GPASettingsSelector *ss);
-static void gpa_ss_printer_modified (GPANode *printer, GPASettingsSelector *ss);
-static void gpa_ss_settings_modified (GPANode *settings, GPASettingsSelector *ss);
-
 static void gpa_ss_rebuild_menu (GPASettingsSelector *ss);
 
 static void gpa_ss_menuitem_activate (GtkWidget *widget, gint index);
@@ -139,45 +135,6 @@ gpa_settings_selector_construct (GPAWidget *widget)
 	return TRUE;
 }
 
-static void
-gpa_ss_root_modified (GPANode *config, GPASettingsSelector *ss)
-{
-	GPANode *printer;
-
-#ifdef GPA_SS_DEBUG
-	g_print ("Root modified\n");
-#endif
-
-	printer = gpa_node_get_path_node (config, "Printer");
-	if (printer != ss->printer) {
-		if (ss->printer) {
-			gpa_node_unref (ss->printer);
-			ss->printer = NULL;
-		}
-		if (printer) {
-			ss->printer = printer;
-		}
-		/* fixme: rebuild menu? maybe only if !printer */
-	} else {
-		if (printer) gpa_node_unref (printer);
-	}
-
-	/* fixme: this is here, because we create/destroy settings */
-	gpa_ss_rebuild_menu (ss);
-}
-
-static void
-gpa_ss_printer_modified (GPANode *printer, GPASettingsSelector *ss)
-{
-	gpa_ss_rebuild_menu (ss);
-}
-
-static void
-gpa_ss_settings_modified (GPANode *settings, GPASettingsSelector *ss)
-{
-	gpa_ss_rebuild_menu (ss);
-}
-
 static void
 gpa_ss_rebuild_menu (GPASettingsSelector *ss)
 {
@@ -283,13 +240,16 @@ gpa_ss_menuitem_activate (GtkWidget *widget, gint index)
 
 	g_free (value);
 #else
-	g_print ("Settings %d activated\n", index);
+	g_print ("Settings selector (%d) activated\n", index);
 #endif
 }
 
 static void
 gpa_ss_add_settings_activate (GtkWidget *widget, GPAWidget *gpaw)
 {
+#ifdef __GNUC__
+#warning Implement add settings
+#endif	
 #if 0
 	GtkWidget *apd;
 
@@ -297,7 +257,7 @@ gpa_ss_add_settings_activate (GtkWidget *widget, GPAWidget *gpaw)
 
 	gtk_widget_show (apd);
 #else
-	g_print ("Add settings\n");
+	g_print ("Implement me. Add settings\n");
 #endif
 }
 
